/*
 *  memon/apoyo.h
 *
 *  Fernando Prez Costoya
 *
 */

/*
 *
 * Fichero de cabecera que contiene macros de utilidad para manejar la
 * mscara de proteccin y el prototipo de ejecutar_programa y tratar_SEGV
 *
 *      NO SE DEBE MODIFICAR
 *
 */

#ifndef _APOYO_H
#define _APOYO_H


/* Macros que facilitan el manejo de la mscara de proteccin */

/* PROT_ISSET:	Permite comprobar si en la proteccion est activo un 
		determinado permiso.
		Ejemplo: comprueba si en la mscara est activo
		el permiso de escritura
 
			if (PROT_ISSET(PROT_WRITE,prot))
*/

#define PROT_ISSET(prot_flag, proteccion) (prot_flag & proteccion)

/* PROT_SET:	Activa un determinado permiso en una mscara de
		proteccin devolviendo la mscara actualizada.
		Ejemplo: Aade a una mscara el permiso de escritura

			prot=PROT_SET(PROT_WRITE,prot);
*/
#define PROT_SET(prot_flag, proteccion) (proteccion | prot_flag )


/* PROT_CLR:	Desactiva un determinado permiso en una mscara de
		proteccin devolviendo la mscara actualizada.
		Ejemplo: Elimina en una mscara el permiso de escritura

			prot=PROT_CLR(PROT_WRITE,prot);
*/
#define PROT_CLR(prot_flag, proteccion) (proteccion & ~prot_flag)


/* Rutina que establece qu funcin realiza el tratamiento de SEGV */
void tratar_SEGV(void (*funcion)(void *));

/* Rutina que arranca la ejecucion del programa */
void ejecutar_programa(int argc, char **args);

#endif /* _APOYO_H */

