/*
 *  memon/mapa.c
 *
 *  Fernando Prez Costoya
 *
 */

/*
 *
 * Fichero que contiene operaciones relacionadas con la gestin del
 * mapa del proceso (sus regiones y las pginas contenidas en las mismas). 
 * Puede aadir las funciones que considere oportuno. 
 *
 *	SE DEBE MODIFICAR PARA INCLUIR LA FUNCIONALIDAD PEDIDA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <sys/mman.h>

#include "mapa.h"
#include "apoyo.h"
#include "marcos.h"


/* Variable global que contiene el tamao de la pgina en el sistema */
extern long tam_pagina;

/* Nmero mximo de regiones previsto */
#define MAX_REG 16

/* Definicin de la tabla de regiones */
entrada_tabla_regiones tabla_regiones[MAX_REG];

/* Prototipos de las funciones internas */

/* Reserva una regin libre */
static int reservar_region();

/* Obtiene la regin a a partir de su direccin inicial */
static int region(const void *dir);

/* Crea una tabla de pginas */
static entrada_tabla_paginas *creacion_tabla_paginas(
				entrada_tabla_regiones *reg);

/* Elimina una tabla de pginas */
static void eliminar_tabla_paginas(const entrada_tabla_regiones *reg);

/* Inicia una entrada de la tabla de pginas. DEBE COMPLETARSE */
static void iniciar_entrada_tpag(entrada_tabla_regiones *reg,
				void *dir, entrada_tabla_paginas *entrada);

/* Libera una entrada de la tabla de pginas. DEBE COMPLETARSE */
static void liberar_entrada_tpag(entrada_tabla_paginas *entrada);

/* FUNCIONES INVOCADAS DESDE EL MDULO DE APOYO */

/* nos informan de que se ha creado una regin
	dir: direccin de comienzo de la regin
	inodo: fichero al que est vinculada (0 si es annima)
	prot: permisos de acceso a la regin
	tamano: tamao de la regin
	compartida: es una regin de tipo compartida?
*/
void creacion_region(void *dir, int inodo, int prot,
			int tamano, int compartida)
{
        int reg;

	reg=reservar_region();
	/* Compruebo que no se han agotado las regiones */
	assert(reg!=-1);

        tabla_regiones[reg].npags=tamano/tam_pagina;
        tabla_regiones[reg].dir_inicial=dir;
        tabla_regiones[reg].ino=inodo;
        tabla_regiones[reg].prot=prot;
        tabla_regiones[reg].compartida=compartida;
	tabla_regiones[reg].tabla_paginas=
		creacion_tabla_paginas(&tabla_regiones[reg]);
}

/* nos informan de que se ha eliminado una regin
	dir: direccin de comienzo de la regin
*/
void eliminacion_region(const void *dir){
	int reg;

	reg=region(dir);
	/* Compruebo que la regin exsite */
	assert(reg!=-1);
	
        tabla_regiones[reg].usada=0;
	eliminar_tabla_paginas(&tabla_regiones[reg]);
}

/* nos informan de que ha cambiado el tamao de una regin
        dir: direccin de comienzo de la regin
        tamano: nuevo tamao de la regin
*/
void cambio_tam_region(void *dir, int tamano){
	/* DEBE COMPLETARSE */
        return;
}

/* Funciones auxiliares que a partir de la direccin de una entrada
   obtienen su posicin en la tabla */

int regnum(entrada_tabla_regiones *reg) {
	return reg-tabla_regiones;
}
int pagnum(entrada_tabla_paginas *pag) {
	return pag-pag->region->tabla_paginas;
}

/* Funcin auxiliar que a partir de un nmero de pgina de una
regin devuelve la direccin de su entrada en la tabla */
entrada_tabla_paginas *entrada_pagina(int regnum, int pagnum) {
	return &(tabla_regiones[regnum].tabla_paginas[pagnum]);
}

/* FUNCIONES INTERNAS */

/* Reserva una regin libre */
static int reservar_region() {
	int i;

        for (i=0; i<MAX_REG; i++)
                if (!(tabla_regiones[i].usada)) {
                        tabla_regiones[i].usada=1;
                        return i;
		}
	return -1;
}

/* Obtiene la regin a partir de su direccin inicial */
static int region(const void *dir){
        int i;

        for (i=0; i<MAX_REG; i++)
                if ((tabla_regiones[i].usada) &&
		   (tabla_regiones[i].dir_inicial==dir))
                        return i;
	return -1;
}



/* Crea una tabla de pginas */
static entrada_tabla_paginas *creacion_tabla_paginas(
					entrada_tabla_regiones *reg) {
	int i;
	char *dir;
	int npags;
	entrada_tabla_paginas *tpag;

	dir=reg->dir_inicial;
	npags=reg->npags;
	tpag=malloc(npags * sizeof(entrada_tabla_paginas));
	for (i=0 ; i<npags; i++, dir+=tam_pagina)
		iniciar_entrada_tpag(reg, dir, &tpag[i]);

	return tpag;
}

/* Elimina una tabla de pginas */
static void eliminar_tabla_paginas(const entrada_tabla_regiones *reg) {
	int i;
	int npags;
	entrada_tabla_paginas *tpag;
	int reg_comp;

	reg_comp=reg->compartida;
	npags=reg->npags;
	tpag=reg->tabla_paginas;
	if (!reg_comp)
		for (i=0; i<npags; i++)
			liberar_entrada_tpag(&tpag[i]);
	free(tpag);
}

/* Inicia una entrada de la tabla de pginas */
static void iniciar_entrada_tpag(entrada_tabla_regiones *reg,
				void *dir,
				entrada_tabla_paginas *entrada){

	entrada->dir_inicial=dir;
	entrada->region=reg;

	/* DEBE COMPLETARSE */

}

/* Libera una entrada de la tabla de pginas */
static void liberar_entrada_tpag(entrada_tabla_paginas *entrada){

	/* DEBE COMPLETARSE */
}

/* PUEDE INCLUIR LAS FUNCIONES QUE CONSIDERE OPORTUNO */

