/*
 *  memon/mapa.h
 *
 *  Fernando Prez Costoya
 *
 */

/*
 *
 * Fichero de cabecera que contiene prototipos exportados por mapa.c
 * y la definicin de la entrada de la tabla de regiones y de pginas.
 *
 *
 *      SE DEBE MODIFICAR PARA COMPLETAR LA DEFINICIN DE LA ENTRADA DE
 *	LA TABLA DE PGINAS. ADEMS, SE PUEDE MODIFICAR PARA INCLUIR
 *      FUNCIONES ADICIONALES (como, por ejemplo, para activar el
 *	de modificado o dada una direccin encontrar la pgina)
 *
 */

#ifndef _MAPA_H
#define _MAPA_H


/* Prototipos de las funciones usadas por el entorno de apoyo
   para notificar los cambios en el mapa del proceso.
	NO SE DEBEN MODIFICAR
*/

/* nos informan de que se ha creado una regin
        dir: direccin de comienzo de la regin
        inodo: fichero al que est vinculada (0 si es annima)
        prot: permisos de acceso a la regin
        tamano: tamao de la regin
	compartida: es una regin de tipo compartida?
*/
void creacion_region(void *dir, int inodo, int prot,
			int tamano, int compartida);
/* nos informan de que ha cambiado el tamao de una regin
        dir: direccin de comienzo de la regin
        tamano: nuevo tamao de la regin
*/
void cambio_tam_region(void *dir, int tamano);
/* nos informan de que se ha eliminado una regin
        dir: direccin de comienzo de la regin
*/
void eliminacion_region(const void *dir);

/* Definiciones de las entradas de la tabla de pginas y de regiones */

struct entrada_tabla_regiones_t;

/* Tipo que define una entrada en la tabla de pginas */
typedef struct {
        void *dir_inicial;	/* direccin inicial de la pgina */
	struct entrada_tabla_regiones_t *region; /* regin a la que pertenece */
} entrada_tabla_paginas;


/* Tipo que define una entrada en la tabla de regiones */
typedef struct entrada_tabla_regiones_t {
        int usada;
        void *dir_inicial;	/* direccin inicial de la regin */
        int ino;		/* inodo del fichero o 0 si annima */
        int prot;		/* proteccin de la regin */
        int npags;		/* tamao en pginas */
        int compartida;		/* regin compartida? */
        entrada_tabla_paginas *tabla_paginas;/* Tabla de pginas de la regin */
} entrada_tabla_regiones;

/* Funciones auxiliares que a partir de la direccin de una entrada
   obtienen su posicin en la tabla correspondiente */
int regnum(entrada_tabla_regiones *reg);

int pagnum(entrada_tabla_paginas *pag);

/* Funcin auxiliar que a partir de un nmero de pgina de una
regin devuelve la direccin de su entrada en la tabla */
entrada_tabla_paginas *entrada_pagina(int regnum, int pagnum);

#endif /* _MAPA_H */
